// Generator DDS z wobulatorem - 3rd ed.
// AVT-5980 - by Adam Sobczyk SQ5RWQ
// Definicje funkcji pomocniczych


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "AVT-5980-lcd.h"
#include "AVT-5980-functions.h"


// INT1 external interrupt init
void ext_int1_init(void)
{
  EICRA|=_BV(ISC11); EICRA&=_BV(ISC10);   // INT1 triggered on falling edge
  EIMSK|=_BV(INT1);                       // enable INT1
  sei();                                  // global interrupt flag set
}

// INT1 external interrupt service routine
ISR(INT1_vect)
{
	extern char enc_new, enc_old;					// biezacy (nowo odczytany) oraz poprzedni stan enkodera
	extern char enc_state;         				// kombinacja  enc_new (b1b0) oraz enc_old (b3b2) w jednej zmiennej
	extern char enc_todo;      						// wynik analizy robionej w przerwaniu: co zadzialo sie z enkoderem: 1 -> w prawo, 2 -> w lewo

	enc_old = enc_new;  									// zapamietanie poprzedniej wartosci zmiennej enc_new
	enc_new = PINB & 0b00000011;   				// tylko piny PB.1 i PB.0 maja znaczenie
	enc_state = enc_new | (enc_old<<2); 	// bity b3..b0 sa kombinacja nowego i poprzedniego stanu wyjsc enkodera na pinach PB.1..PB.0
    
	switch(enc_state)   // analiza i interpretacja zmian stanow wyjsc enkodera, teoretycznie mozliwych w zakresie 0..15
	{
		case 1:     enc_todo=1; break;  		// PRAWO
		case 4:     enc_todo=2; break;  		// LEWO
	}
}

// uruchomienie chip'u AD9850 w module DDS
void dds_init(void)
{
	PORTC |= _BV(PC2); 		// Set Dds_reset
	PORTC &= ~_BV(PC2); 	// Reset Dds_reset
	PORTC |= _BV(PC5);		// Set Dds_w_clk
	PORTC &= ~_BV(PC5);		// Reset Dds_w_clk
	PORTC |= _BV(PC4);		// Set Dds_fq_ud
	PORTC &= ~_BV(PC4);		// Reset Dds_fq_ud
}

// ustawienie czestotliwosci pracy modulu DDS
void dds_freq_set(uint32_t Fint)
{
	float Fgen;				// czestotliwosc pracy modulu DDS, rzutowana na typ float celem zwiekszenia dokladnosci przeliczen
	float Freq_d;			// czestotliwosc pracy modulu DDS-a (po przeliczeniu na przeskalowana wartosc float)
	uint32_t Freq;		// czestotliwosc pracy modulu DDS w postaci zaokraglonej wartosci calkowitej (juz do zaladowania do modulu DDS)

	Fgen = (float) Fint;				// rzutowanie
	Freq_d = F_const * Fgen;		// skalowanie
	Freq = Freq_d + 0.5;				// zaokraglenie i rzutowanie

	// Transfer szeregowy "bit po bicie" zmiennej Freq (od LSB) do portu DDS_DATA modulu DDS
	// Wpis odbywa sie na narastajacym zboczu na wejsciu DDS_WCLK modulu DDS
	PORTC &= ~_BV(PC5);				// DDS_WCLK=0 (taktowanie)
	for(uint8_t i=0; i<32; i++)
	{
	 	if((Freq>>i)&1)
	    PORTC |= _BV(PC3);		// DDS_DATA=1 (dana)
   	else
 	    PORTC &= ~_BV(PC3);		// DDS_DATA=0 (dana)

		PORTC |= _BV(PC5);			// DDS_WCLK=1 (taktowanie)
		PORTC &= ~_BV(PC5);			// DDS_WCLK=0 (taktowanie)
	}

	// Transfer szeregowy "bit po bicie" zmiennej sterujacej Init (od LSB) do portu DDS_DATA modulu DDS
	// Wpis odbywa sie na narastajacym zboczu na wejsciu DDS_WCLK modulu DDS; Init=0x00, wiec wysuwamy same zera
	PORTC &= ~_BV(PC5);				// DDS_WCLK=0 (taktowanie)
	PORTC &= ~_BV(PC3);				// DDS_DATA=0 (dana)
	for(uint8_t i=0; i<8; i++)
	{
		PORTC |= _BV(PC5);			// DDS_WCLK=1 (taktowanie)
		PORTC &= ~_BV(PC5);			// DDS_WCLK=0 (taktowanie)
	}

	// Update F pracy modulu DDS
	PORTC |= _BV(PC4);				// Set DDS_FQ_UD
	PORTC &= ~_BV(PC4);				// Reset DDS_FQ_UD
}

// wyswietlanie aktualnej Fg, Fl albo Fh (F_which=1,2,3)
void F_show(uint8_t F_which, uint32_t Fg)
{
	extern char linebuf[17];		// bufor znakowy dla wyświetlacza LCD 16x2 (jedna linia)
	char tb[13] = "";						// bufor znakowy dla sformatowania Fg: max.: 8 cyfr + 2 kropki + spacja + 'k'/'M' + '\0'
	
	ltoa(Fg, tb, 10);						// konwersja Fg na string

	switch(strlen(tb))																					// warunkowe formatowanie wyswietlania
	{
		case 1 : {tb[1]=' '; tb[2]='\0';}; break;									// format: "x " [Hz]
		case 2 : {tb[2]=' '; tb[3]='\0';}; break;									// format: "xx " [Hz]
		case 3 : {tb[3]=' '; tb[4]='\0';}; break;									// format: "xxx " [Hz]
		case 4 : {tb[4]=tb[3]; tb[3]=tb[2]; tb[2]=tb[1];
			tb[1]='.'; tb[5]=' '; tb[6]='k'; tb[7]='\0';}; break;		// format: "x.xxx k" [Hz]
		case 5 : {tb[5]=tb[4]; tb[4]=tb[3]; tb[3]=tb[2];
			tb[2]='.'; tb[6]=' '; tb[7]='k'; tb[8]='\0';}; break;		// format: "xx.xxx k" [Hz]
		case 6 : {tb[6]=tb[5]; tb[5]=tb[4]; tb[4]=tb[3];
			tb[3]='.'; tb[7]=' '; tb[8]='k'; tb[9]='\0';}; break;		// format: "xxx.xxx k" [Hz]
		case 7 : {tb[7]=tb[6]; tb[6]=tb[5]; tb[5]=tb[4];
			tb[4]=tb[3]; tb[3]=tb[2]; tb[2]=tb[1];
			tb[1]='.'; tb[8]=' ';
			tb[9]='M'; tb[10]='\0';}; break;												// format: "x.xxxxxx M" [Hz]
		case 8 : {tb[8]=tb[7]; tb[7]=tb[6]; tb[6]=tb[5];
			tb[5]=tb[4]; tb[4]=tb[3]; tb[3]=tb[2];
			tb[2]='.'; tb[9]=' ';
			tb[10]='M'; tb[11]='\0';}; break;												// format: "xx.xxxxxx M" [Hz]
		default : break;
	}
	
	switch(F_which)																							// warunkowe wyswietlanie
	{
		case 1 :																									// Fg w trygie generatora
			{snprintf(linebuf, sizeof(linebuf), "Fg=%sHz         ", tb);
			InstLcd(LCDLINE1); PutsLcd(linebuf);}; break;						// LCD out
		case 2 :																									// Fl w trybie wobulatora
			{snprintf(linebuf, sizeof(linebuf), "Fl=%sHz         ", tb);
			InstLcd(LCDLINE1); PutsLcd(linebuf);}; break;						// LCD out
		case 3 :																									// Fh w trybie wobulatora
			{snprintf(linebuf, sizeof(linebuf), "Fh=%sHz         ", tb);
			InstLcd(LCDLINE2); PutsLcd(linebuf);}; break;						// LCD out
		default : break;
	}
}

// wyswietlanie aktualnego dFg
void dFg_show(uint32_t dFg)
{
	extern char linebuf[17];		// bufor znakowy dla wyświetlacza LCD 16x2 (jedna linia)
	
	if(dFg<1000) snprintf(linebuf, sizeof(linebuf), "dFg=%ld Hz            ", dFg);								// dFg<1kHz
	else if(dFg<1000000) snprintf(linebuf, sizeof(linebuf), "dFg=%ld kHz           ", dFg/1000);	// 1kHz<=dFg<1MHz
	else snprintf(linebuf, sizeof(linebuf), "dFg=%ld MHz           ", dFg/1000000);								// dFg>=1MHz
	InstLcd(LCDLINE1); PutsLcd(linebuf);																													// LCD out
	snprintf(linebuf, sizeof(linebuf), "----------------");																				// wypelnienie LCDLINE2
	InstLcd(LCDLINE2); PutsLcd(linebuf);																													// LCD out
}

// wyswietlanie aktualnej Fw
void Fw_show(uint16_t Fw)
{
	extern char linebuf[17];		// bufor znakowy dla wyświetlacza LCD 16x2 (jedna linia)
	
	snprintf(linebuf, sizeof(linebuf), "Fw=%d Hz             ", Fw);
	InstLcd(LCDLINE1); PutsLcd(linebuf);															// LCD out
	snprintf(linebuf, sizeof(linebuf), "----------------");						// wypelnienie LCDLINE2
	InstLcd(LCDLINE2); PutsLcd(linebuf);															// LCD out
}

// inicjalizacja ADC
void adc_init(void)
{
    ADMUX = (1<<REFS0);																		// AREF = AVcc
    ADCSRA = (1<<ADEN)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);	// ADC Enable & prescaler=128 -> Fadc = 16000000/128 = 125000 Hz
}

// odczyt wartosci ADC z wybranego kanalu w zakresie 0..7
uint16_t adc_read(char channel)
{
  channel &= 0b00000111;							// ograniczenie zakresu #-ow kanalow do 0..7
  ADMUX = (ADMUX & 0xF0) | channel;		// reset 4-ech najmlodszych bitow (MUX3..0) oraz wpisanie #-u kanalu
  ADCSRA |= (1<<ADSC);								// wpisanie 1 do bitu ADSC - rozpoczyna pojedyncza konwersje
  while(ADCSRA & (1<<ADSC));					// oczekiwanie na zakonczenie konwersji (ADSC==0)
  return (ADC);
}

// pomiar aktualnego Vout
float Vout_measure(void)
{
  return 0.004882813 * ((float) adc_read(0));		// pomiar ADC0 przeskalowany na [Vpp] (5.0V/1024=0.004882813V)
}

// wyswietlanie aktualnego Vout
void Vout_show(float V)
{
	extern char linebuf[17];		// bufor znakowy dla wyświetlacza LCD 16x2 (jedna linia)
	char tb[5] = ""; 						// bufor znakowy na sformatowany liczbowy wynik pomiaru
	
	if(V<=0.02) snprintf(linebuf, sizeof(linebuf), "Vout<=0.1 Vpp       ");
	else
	{
		// korekta wartosci Vout wg uzyskanej pomiarowo (@: Fgen=1MHz, 0.1V<Vout<4.0V) interpolacyjnej formuly:
		// f(x)=-0.0586143037452043*x^2 + 1.18569526919539*x + 0.0887991642390158
		V = (1.18569526919539-0.0586143037452043*V)*V+0.0887991642390158;		// zdekomponowana formula - dla przyspieszenia obliczen
		if(V>=4.00) snprintf(linebuf, sizeof(linebuf), "Vout>=4.00 Vpp      ");
		else
		{
			itoa( (int) (100*V), tb, 10 );	// V*100 rzutowane na l-be calkowita i zamienione na string (zaokraglenie i skrocenie tekstu)
			tb[4]='\0';											// ustawienie znacznika konca tekstu na koncu bufora
			switch(strlen(tb))																					// formatowanie warunkowe
			{
				case 1 : {tb[3]=tb[0]; tb[0]='0'; tb[2]='0';}; break;			// format: "0.0x" [ Vpp   ]
				case 2 : {tb[3]=tb[1]; tb[2]=tb[0]; tb[0]='0';}; break;		// format: "0.xx" [ Vpp   ]
				case 3 : {tb[3]=tb[2]; tb[2]=tb[1];}; break;							// format: "x.xx" [ Vpp   ]
				default : break;
			}
			tb[1]='.';											// ustawienie kropki dziesietnej
			snprintf(linebuf, sizeof(linebuf), "Vout=%s Vpp   ", tb);
		}
	}
	InstLcd(LCDLINE2); PutsLcd(linebuf);	// LCD out
}

// konfiguracja i inicjalizacja PWM/OCR1B/PB.2
void pwm_init(void)
{
	DDRB  &= ~(1<<PB2);	PORTB &= ~(1<<PB2);						// tymczasowe ustawienie PB.2 jako IN (bez pull-up-u)
	TCCR1A |= (1<<COM1A1)|(1<<COM1B1) ;								// Clear OC1A/OC1B on Compare Match, set OC1A/OC1B at BOTTOM (non-inverting mode)
	TCCR1A |= (1<<WGM10); TCCR1B |= (1<<WGM12);				// Mode 5: Fast PWM, 8-bit
	TCCR1B |= (1<<CS10);															// clk_I_O/1 (No prescaling)
	DDRB |= (1<<PB2);																	// docelowe ustawienie PB.2 jako OUT
}

// konfiguracja timera TC0
void tc0_config(void)
{
	TCCR0B &= ~(1<<CS02) & ~(1<<CS01) & ~(1<<CS00);		// brak zrodla zegara (TC0 zatrzymany)
	TCCR0A = (1<<WGM01);															// tryb CTC, piny OC0A/B odlaczone
	TIMSK0 = 0;																				// przerwania od TC0 zabronione
	OCR0B = 0;																				// prewencyjne wyzerowanie rejestru OCR0B (nie bedzie uzywany w trybie CTC)
}

// start timera TC0
void tc0_start(void)
{
	// pozadana wartosc rejestru OCR0A powinna zostac ustawiona przed startem TC0
	TCNT0 = 0;													// kasowanie licznika TC0
	TIFR0 |= (1<<OCF0A);								// czyszczenie flagi przerwania OCF0A w rejestrze IFR przez wpisanie logicznej 1-ki
	TCCR0B |= (1<<CS01) | (1<<CS00);		// uruchomienie zegara TC0 poprzez ustawienie clk_I_O/64 w preskalerze (4us na 1 pelny cykl TC0)
}

// zatrzymanie timera TC0
void tc0_stop(void)
{
	TCCR0B &= ~(1<<CS02) & ~(1<<CS01) & ~(1<<CS00);		// brak zrodla zegara (TC0 zatrzymany)
}

