﻿#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include "AVT-5980-lcd.h"

#define INSTR 0x0
#define DANE 0x1

/********** Podłączenie LCD ************
Wyświetlacz            AVR
E                     PD2
RW                    PD1
RS                    PD0
D0-D3                   -
D4-D7    	PD4-PD7 (odpowiednio zgodnie)
****************************************/

//Definicje pinow dla danych (starszy półbajt portu D)
#define LCDDIRD DDRD
#define LCDOUTD PORTD
#define LCDIND PIND

//Definicje pinow dla sterowania (młodszy półbajt portu D)
#define LCDDIRC DDRD
#define LCDOUTC PORTD

#define LCD_E  _BV(PD2)
#define LCD_RW _BV(PD1)
#define LCD_RS _BV(PD0)

#define E1() LCDOUTC|=LCD_E
#define E0() LCDOUTC&=~LCD_E
#define RW1() LCDOUTC|=LCD_RW
#define RW0() LCDOUTC&=~LCD_RW
#define RS1() LCDOUTC|=LCD_RS
#define RS0() LCDOUTC&=~LCD_RS

//Funkcja no operation
#define nop() asm volatile("nop\n " \
						   "nop\n"::)

//Funkcja zapisujaca dana 4 bitowa -> na starszy półbajt !!!
static void wr4d(unsigned char n)
{
	LCDOUTD &= 0x0F;		// czyszczenie starszego polbajtu
	//LCDOUTD &= 0xF0;			// czyszczenie młodszego półbajtu
	LCDOUTD |= n & 0xF0;		// zapisa starszego polbajtu z liczby "n"
	//LCDOUTD |= n & 0x0F;		// zapis młodszego półbajtu - z liczby "n"
}

// Wysyła daną lub instrukcję - w zależności od wartości parametru "id"
void WriteLcd(unsigned char dana, unsigned char id)
{	
	unsigned char result;
	unsigned int tout = 32000;
	//Sprawdzenie czy jest wolny wyswietlacz
	do
	{
		E0();
		RS0();
		RW1();
		
		LCDDIRD &= 0x0f;	// ustawiamy starszy polbajt jako linie wejsciowe (odczyt)
		//LCDDIRD &= 0xF0;		// ustawiamy młodszy półbajt jako linie wejściowe (odczyt)
		
		LCDOUTD |= 0xf0;	// pull-up na starszy polbajt
		//LCDOUTD |= 0x0F;		// pull-up-y na młodszy półbajt
		
		nop();
		E1();
		nop();
		
		result = LCDIND;		// odczyt stanu	szyny danych wyświetlacza LCD
		
		E0();
		nop();
		nop();
		E1();						//Opusc mlodszy polbajt
		nop();
		nop();
		E0();
		if(--tout == 0) return;
	}
	while(result & 0x80);		// badamy starszy polbajt - czy jest pozadany stan na LCD???
	//while(result & 0x08);		// badamy młodszy półbajt - czy jest pożądany stan LCD ???
	
	LCDDIRD |= 0xF0;		// ustawiamy starszy polbajt jako linie wyjsciowe
	//LCDDIRD |= 0x0F;			// ustawiamy młodszy półbajt jako linie wyjściowe
	
	// Właściwy zapis danej do portu (Interfejs 4 bitowy)
	E0();
	RW0();		
	if(id) RS1();
	else RS0();
	
	wr4d(dana);			//Starszy polbajt
	//wr4d(dana>>4);			// zapis starszego półbajtu danych na szynę danych
	
	E1();
	nop();
	E0();
	
	wr4d(dana<<4);
	//wr4d(dana);			// zapis młodszego półbajtu danych na szynę danych
	
	E1();
	nop();
	E0();
}

//Inicjalizacja modulu LCD
void InitLcd(void)
{
	LCDDIRD |= 0xF0;
	//LCDDIRD |= 0x0F;			// ustawiamy młodszy półbajt jako linie wyjściowe
	
	LCDDIRC |= LCD_E | LCD_RW | LCD_RS;			// aktywacja linii sterujących jako wyjściowe
	LCDOUTC &= ~(LCD_E | LCD_RW | LCD_E);		// zerowanie (reset) linii sterujących
	
	// LCD8BITS      0x30 //0b00110000 //select 8 Bit interface
	wr4d(LCD8BITS);
	//wr4d(LCD8BITS>>4);				// zapis starszego półbajtu danych na szynę danych
	
	E1();
	nop();
	E0();
	_delay_ms(20);
	E1();
	nop();
	E0();
	_delay_ms(10);
	LCDOUTC &= ~(LCD_E | LCD_RW | LCD_E);
	
	// LCD4BITS      0x20 //0b00100000 //select 4 Bit interface
	wr4d(LCD4BITS);
	//wr4d(LCD4BITS>>4);				// zapis starszego półbajtu danych na szynę danych
	
	E1();
	nop();
	E0();
	_delay_ms(20);
	WriteLcd(0x28,INSTR);			// Interfejs 4 bitowy 2 linie -> OK!
	WriteLcd(0x6,INSTR);			// Tryb pracy wyświetlacza LCD
	WriteLcd(0x0c,INSTR);			// Włącz wyświetlacz LCD
	WriteLcd(LCDCLEAR,INSTR);		// Wyczyść wyświetlacz LCD ... :-)
    _delay_ms(20);
}

//Wyswietla string na wyswietlaczu LCD (RAM)
void PutsLcd(char *nap)
{
	while(*nap)
	{
		WriteLcd(*nap,DANE);
	    nap++;
	}
}

//Wyswietla string na wyswietlaczu LCD (ROM)
void PutsLcd_P(PGM_P nap)
{
	while(pgm_read_byte(nap))
	{
		WriteLcd(pgm_read_byte(nap),DANE);
	    nap++;
	}
}
